@@ -5,8 +5,9 @@ use warnings;
 use EV;
 use Net::Curl::Easy qw(CURLE_COULDNT_CONNECT);
 use Net::Curl::Multi qw(CURLMSG_DONE CURL_SOCKET_TIMEOUT /^CURL_POLL_/ /^CURL_CSELECT_/ CURLMOPT_SOCKETFUNCTION CURLMOPT_TIMERFUNCTION);
+use Scalar::Util qw(weaken);
 
-our $VERSION = '0.012';
+our $VERSION = '0.07';
 
 BEGIN {
 	Net::Curl::Multi->can('CURLMOPT_TIMERFUNCTION') or
@@ -17,8 +18,6 @@ BEGIN {
 sub curl_ev {
 	my ($multi) = @_;
 
-	$multi ||= Net::Curl::Multi->new();
-
 	my %finish = (); # easy => finish
 	my %timer  = (); # easy => sub
 
@@ -44,6 +43,8 @@ sub curl_ev {
 
 	my $cb_timeout = sub { $socket_action->(CURL_SOCKET_TIMEOUT) };
 
+	my $add_handle_timer;
+
 	# add one handle and kickstart download
 	my $add_handle = sub {
 		my ($easy, $finish, $timeout) = @_;
@@ -62,7 +63,7 @@ sub curl_ev {
 		# and it could confuse our application -- it would appear to
 		# have finished before it started.
 
-		EV::timer 0, 0, $cb_timeout;
+		$add_handle_timer = EV::timer 0, 0, $cb_timeout;
 
 		$multi->add_handle($easy);
 
@@ -70,7 +71,7 @@ sub curl_ev {
 			$timer{$easy} = EV::timer $timeout, 0, sub {
 				delete $timer{$easy};
 				$multi->remove_handle($easy);
-				$finish{$easy}->($easy, CURLE_COULDNT_CONNECT);
+				$finish{$easy}->($easy, bless( do{\(my $o = CURLE_COULDNT_CONNECT)}, 'Net::Curl::Easy::Code' ));
 				delete $finish{$easy};
 			};
 		}
@@ -129,7 +130,8 @@ sub curl_ev {
 			$timer = EV::timer 10, 10, $cb_timeout;
 		} else {
 			# This will trigger timeouts if there are any.
-			$timer = EV::timer $timeout_ms / 1000, 0, $cb_timeout;
+			my $t = $timeout_ms / 1000;
+			$timer = EV::timer $t, $t, $cb_timeout;
 		}
 
 		return 1;
@@ -138,6 +140,8 @@ sub curl_ev {
 	$multi->setopt(CURLMOPT_SOCKETFUNCTION, $cb_socket);
 	$multi->setopt(CURLMOPT_TIMERFUNCTION,  $cb_timer);
 
+	weaken $multi;
+
 	return $add_handle;
 }
 
@@ -201,5 +205,6 @@ at your option, any later version of Perl 5 you may have available.
 
 L<EV>
 L<Net::Curl>
+L<http://search.cpan.org/~syp/Net-Curl/lib/Net/Curl/examples.pod#Multi::Event>
 
 =cut
